from __future__ import print_function

"""
Handle configuration files with variables.
"""

"""
RCSid:
	$Id: conf.py,v 1.21 2022/05/22 16:53:29 sjg Exp $

	@(#) Copyright (c) 2012-2021 Simon J. Gerraty

	This file is provided in the hope that it will
	be of use.  There is absolutely NO WARRANTY.
	Permission to copy, redistribute or otherwise
	use this file is hereby granted provided that 
	the above copyright notice and this notice are
	left intact. 
      
	Please send copies of changes and bug-fixes to:
	sjg@crufty.net

"""

import sys
import os
import vars

def host_target():
    u = os.uname()
    return '{0}{1}-{2}'.format(u[0].lower(), u[2].split('.')[0], u[-1])

def stringConfig(s, conf={}):
    """Add key value pairs to conf from a string

    string = one and two
    string := one two
    string ?= maybe
    string += three
    list [=] one,two,three
    list [?=] maybe,more
    list [+=] four,five

    values and indeed variable names may reference variables
    that correspond to keys in conf or the environment.
    For example::

    	Home ?= ${HOME:U/homes/${USER}}
    

    Note ``:=`` and ``=`` are equivalent.

    All variable references are resolved before storing in ``conf``.
    List values are split on ``conf['list_split']`` or ``,``
    """

    if '.Vars' not in conf:
        vs = conf['.Vars'] = vars.Vars(conf)
    else:
        vs = conf['.Vars']
    splits = conf.get('list_split', ',')
    if s.find('[+=]') > 0:
        # value is a list to append
        k,v = s.split('[+=]', 1)
        k = vs.subst(k).strip()
        val = vs.subst(v).strip().split(splits)
        if not k in conf:
            conf[k] = val
        else:
            conf[k] += val
            val = conf[k]
        vs.set(k, val)
    elif s.find('[?=]') > 0:
        # value is a list
        k,v = s.split('[?=]', 1)
        k = vs.subst(k).strip()
        if k not in conf:
            conf[k] = val = vs.subst(v).strip().split(splits)
            vs.set(k, val)
    elif s.find('[=]') > 0:
        # value is a list
        k,v = s.split('[=]', 1)
        k = vs.subst(k).strip()
        conf[k] = val = vs.subst(v).strip().split(splits)
        vs.set(k, val)
    elif s.find('+=') > 0:
        k,v = s.split('+=', 1)
        k = vs.subst(k).strip()
        if not k in conf:
            conf[k] = val = vs.subst(v).strip()
        else:
            conf[k] += vs.subst(v).strip()
            val = conf[k]
        vs.set(k, val)
    elif s.find('?=') > 0:
        k,v = s.split('?=', 1)
        k = vs.subst(k).strip()
        if k not in conf:
            conf[k] = val = vs.subst(v).strip()
            vs.set(k, val)
    elif s.find(':=') > 0:
        k,v = s.split(':=', 1)
        k = vs.subst(k).strip()
        conf[k] = val = vs.subst(v).strip()
        vs.set(k, val)
    elif s.find('=') > 0:
        k,v = s.split('=', 1)
        k = vs.subst(k).strip()
        conf[k] = val = vs.subst(v).strip()
        vs.set(k, val)
    return conf

def loadConfig(name, conf={}):
    """Add key value pairs from a file into conf

    After stripping trailing white-space, 
    each line in the config file is parsed by stringConfig_

    We set 'config' to the path of the config file and
    'config_dir' to the directory it resides in.
    """
    if '.Vars' not in conf:
        vs = conf['.Vars'] = vars.Vars(conf)
        vs.set('HOST_TARGET', host_target(), '?=')
    else:
        vs = conf['.Vars']
    cf = open(name, 'r')
    vs.set('config', name)
    vs.set('config_dir', '${config:tA:H}', ':=')
    for line in cf:
        line = line.strip()
        if line.startswith('#'):
            continue
        conf = stringConfig(line, conf)
    return conf

def dumpConfig(conf, file=sys.stdout, internals=False):
    for k,v in conf.items():
        if not internals and k.startswith('.'):
            continue
        print('{} = {}'.format(k,v), file=file)
    
if __name__ == '__main__':
    import getopt
    import sys

    opts,args = getopt.getopt(sys.argv[1:], 'dc:')
    debug = 0
    conf = {}
    for o,a in opts:
        if o == '-d':
            debug += 1
        elif o == '-c':
            conf['debug'] = debug
            conf = loadConfig(a, conf)

    for a in args:
        if a.find('=') > 0:
            conf = stringConfig(a, conf)

    dumpConfig(conf)
